(*
 * <COPYRIGHT>
 * Copyright 2005 Gerd Stolpmann
 *
 * <GPL>
 * This file is part of UMLMON.
 *
 * UMLMON is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * UMLMON is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with UMLMON; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * </>
 *)

(* $Id: linux.ml 187 2005-10-14 10:54:15Z gerd $ *)

let catch_unix_error f arg =
  try f arg
  with
      Unix.Unix_error(err,_,errarg) ->
        let errarg' = if errarg <> "" then ": " ^ errarg else errarg in
        raise(Sys_error(Unix.error_message err ^ errarg'))
;;


module Privilege = struct
  external initgroups : string -> int -> unit = "unix_initgroups"
  external getgrouplist : string -> int -> int array = "unix_getgrouplist"
end


module Cap = struct
  (* external drop_privileges : unit -> unit = "unix_drop_privileges" *)
end


module Sysconf = struct
  external pagesize : unit -> int = "misc_sysconf_pagesize"
end


module Pty = struct
  external grantpt : Unix.file_descr -> unit = "unix_grantpt"
  external unlockpt : Unix.file_descr -> unit = "unix_unlockpt"
  external ptsname : Unix.file_descr -> string = "unix_ptsname"
end


module File = struct
  external realpath : string -> string = "unix_realpath"

  type node_type = 
    | S_IFREG 
    | S_IFCHR of int  (* major + minor *)
    | S_IFBLK of int  (* major + minor *)
    | S_IFIFO
    | S_IFSOCK

  external mknod : string -> int -> node_type -> unit = "unix_mknod"

  let copy origname copyname =
    let do_copy =
      catch_unix_error
	(fun () ->
           let b = String.create 4096 in
           let origfd = open_in_bin origname in
           try
(*
             if remove && Sys.file_exists copyname then
               Sys.remove copyname;
 *)
             let copyfd = open_out_bin copyname in
             ( try
		 let eof = ref false in
		 while not !eof do
                   let n = input origfd b 0 (String.length b) in
                   if n=0 then eof := true;
                   output copyfd b 0 n
		 done;
		 let p =
                   (Unix.fstat (Unix.descr_of_in_channel origfd)).Unix.st_perm in               Unix.fchmod (Unix.descr_of_out_channel copyfd) (p land 0o777);
		 close_in origfd;
		 close_out copyfd;
               with
                   error ->
                     close_out copyfd; raise error
             )
         with
             error ->
               close_in origfd; raise error
	)
    in
    do_copy()
end

module Process = struct
  external set_sched_other : int option -> unit = "ml_set_sched_other"
  external set_sched_rr : int -> unit = "ml_set_sched_rr"
end

module Tun = struct
  external create_tap : Unix.file_descr -> string -> int -> string
    = "tun_create_tap"

  external delete_tap : Unix.file_descr -> string -> unit
    = "tun_delete_tap"
end
